/*
    This file is a part of the SteamWar software.

    Copyright (C) 2020  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.steamwar.core;

import com.comphenix.tinyprotocol.Reflection;
import com.viaversion.viaversion.api.Via;
import net.md_5.bungee.api.ChatMessageType;
import net.md_5.bungee.api.chat.BaseComponent;
import org.bukkit.Sound;
import org.bukkit.entity.Player;

import java.util.UUID;

public class BountifulWrapper9 implements BountifulWrapper.IBountifulWrapper {

    @Override
    public void playPling(Player player) {
        player.playSound(player.getLocation(), Sound.ENTITY_EXPERIENCE_ORB_PICKUP, 1, 1);
    }

    @Override
    public void sendMessage(Player player, ChatMessageType type, BaseComponent... msg) {
        if(type == ChatMessageType.CHAT && Via.getAPI().getPlayerVersion(player.getUniqueId()) >= 759)
            type = ChatMessageType.SYSTEM;

        player.spigot().sendMessage(type, msg);
    }

    private static final Class<?> dataWatcherObject = Reflection.getClass("{nms.network.syncher}.DataWatcherObject");
    private static final Class<?> dataWatcherRegistry = Reflection.getClass("{nms.network.syncher}.DataWatcherRegistry");
    private static final Class<?> dataWatcherSerializer = Reflection.getClass("{nms.network.syncher}.DataWatcherSerializer");
    private static final Reflection.ConstructorInvoker dataWatcherObjectConstructor = Reflection.getConstructor(dataWatcherObject, int.class, dataWatcherSerializer);
    @Override
    public Object getDataWatcherObject(int index, Class<?> type) {
        return dataWatcherObjectConstructor.invoke(index, Reflection.getField(dataWatcherRegistry, dataWatcherSerializer, 0, type).get(null));
    }

    private static final Class<?> item = Reflection.getClass("{nms.network.syncher}.DataWatcher$Item");
    private static final Reflection.ConstructorInvoker itemConstructor = Reflection.getConstructor(item, dataWatcherObject, Object.class);
    @Override
    public Object getDataWatcherItem(Object dwo, Object value) {
        return itemConstructor.invoke(dwo, value);
    }

    @Override
    public BountifulWrapper.PositionSetter getPositionSetter(Class<?> packetClass, int fieldOffset) {
        Reflection.FieldAccessor<Double> posX = Reflection.getField(packetClass, double.class, fieldOffset);
        Reflection.FieldAccessor<Double> posY = Reflection.getField(packetClass, double.class, fieldOffset+1);
        Reflection.FieldAccessor<Double> posZ = Reflection.getField(packetClass, double.class, fieldOffset+2);

        return (packet, x, y, z) -> {
            posX.set(packet, x);
            posY.set(packet, y);
            posZ.set(packet, z);
        };
    }

    @Override
    public BountifulWrapper.PositionSetter getRelMoveSetter(Class<?> packetClass) {
        Class<?> type = Core.getVersion() > 12 ? short.class : int.class;
        Reflection.FieldAccessor<?> moveX = Reflection.getField(packetClass, "b", type);
        Reflection.FieldAccessor<?> moveY = Reflection.getField(packetClass, "c", type);
        Reflection.FieldAccessor<?> moveZ = Reflection.getField(packetClass, "d", type);

        return (packet, x, y, z) -> {
            moveX.set(packet, (short)(x*4096));
            moveY.set(packet, (short)(y*4096));
            moveZ.set(packet, (short)(z*4096));
        };
    }

    @Override
    public BountifulWrapper.UUIDSetter getUUIDSetter(Class<?> packetClass) {
        Reflection.FieldAccessor<UUID> uuidField = Reflection.getField(packetClass, UUID.class, 0);

        return uuidField::set;
    }
}
