/*
 *   This file is a part of the SteamWar software.
 *
 *   Copyright (C) 2023  SteamWar.de-Serverteam
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Affero General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Affero General Public License for more details.
 *
 *   You should have received a copy of the GNU Affero General Public License
 *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.steamwar.core;

import com.comphenix.tinyprotocol.Reflection;
import de.steamwar.core.Core;
import lombok.experimental.UtilityClass;
import org.bukkit.Bukkit;
import org.bukkit.scheduler.BukkitTask;

import java.util.function.LongSupplier;

@UtilityClass
public class TPSWarpUtils {

    private static long nanoOffset = 0;
    private static long nanoDOffset = 0;
    private static BukkitTask bukkitTask = null;

    static {
        ProtocolWrapper.impl.initTPSWarp(() -> nanoOffset);
    }

    public static void warp(double tps) {
        double d = 50 - (50 / (tps / 20.0));
        nanoDOffset = Math.max(0, (long) (d * 1000000));
        if (nanoDOffset == 0) {
            if (bukkitTask == null) return;
            bukkitTask.cancel();
            bukkitTask = null;
        } else if (bukkitTask == null) {
            bukkitTask = Bukkit.getScheduler().runTaskTimer(Core.getInstance(), () -> nanoOffset += nanoDOffset, 1, 1);
        }
    }

    public static boolean isWarping() {
        return nanoDOffset > 0;
    }
}
