/*
    This file is a part of the SteamWar software.

    Copyright (C) 2021  SteamWar.de-Serverteam

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

package de.steamwar.core;

import org.bukkit.plugin.Plugin;

import java.lang.reflect.InvocationTargetException;

public class VersionDependent {
    private VersionDependent() {}

    public static <T> T getVersionImpl(Plugin plugin) {
        return getVersionImpl(plugin, (new Exception()).getStackTrace()[1].getClassName());
    }

    public static <T> T getVersionImpl(Plugin plugin, String className){
        ClassLoader loader = plugin.getClass().getClassLoader();
        for(int version = Core.getVersion(); version >= 8; version--) {
            try {
                return ((Class<? extends T>) Class.forName(className + version, true, loader)).getDeclaredConstructor().newInstance();
            } catch (InstantiationException | IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
                throw new SecurityException("Could not load version dependent class", e);
            } catch (ClassNotFoundException e) {
                // try next version
            }
        }
        throw new SecurityException("Unable to find version dependent implementation for " + className);
    }
}
