/*
 * This file is a part of the SteamWar software.
 *
 * Copyright (C) 2021  SteamWar.de-Serverteam
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package de.steamwar.inventory;

import de.steamwar.core.Core;
import de.steamwar.sql.SchematicNode;
import org.bukkit.entity.Player;

public interface SchematicSelectorInjectable {

    SchematicSelectorInjectable DEFAULT = new SchematicSelectorInjectable() {};

    default String createTitle(Player player) {
        return Core.MESSAGE.parse("SCHEM_SELECTOR_TITLE", player);
    }

    default void onSelectorCreate(SchematicSelector selector) {}

    default void onListRender(SchematicSelector selector, SWListInv<SchematicNode> inv, SchematicNode parent) {}

    default void onFilterRender(SchematicSelector selector, SWInventory inventory) {}

    default void onFilterApply(SchematicSelector selector) {}

    default boolean onFolderCreate(SchematicSelector selector, String name) {return true;}

    default void onNodeFilter(SchematicSelector selector, SchematicNode node) {}

    default void onSelectorOpen(SchematicSelector selector, OpenFrom from) {}

    default AnvilInvCloseAction onAnvilInvCloseAction(SchematicSelector selector) {return AnvilInvCloseAction.CLOSE;}

    enum OpenFrom {
        FRESH,
        REOPEN
    }

    enum AnvilInvCloseAction {
        CLOSE,
        REOPEN
    }
}
